/******************************************************************************
FILE:				MKeyboard.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define an abstract Minitel keyboard module.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/21 <PJB> Creation.
	1993/12/23 <PJB> Removed some keyboard codes.
	1994/01/04 <PJB> Added 1/4, 1/2 and 3/4 codes.
	1993/01/06 <PJB> Plainly use UNICODE.
	1994/02/22 <PJB> Added className method.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1994
	All Rights Reserved.
******************************************************************************/
#ifndef __MKeyboard__
#define __MKeyboard__
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H
#include MMODULE_HH
class MProtocol;	//#include MPROTOCOL_HH

	enum {
	// Minitel key codes:
	// It is intended to correspond to the UNICODE encodings. 
	// Codes from 0x00 to 0x7F for the standard ASCII keys.
	// (the meaning of control characters is that of Videotex).

		// NOTE: The accented-letters encodings will change when we know more
		//       about UNICODE.
		// For any letter l in 'A'..'Z','a'..'z', and any MKeyboard_accent a, 
		// compute the unicode as:  u = a + ( l & 0x3f ).
		// A MKeyboard_accent can be given alone and then stand for itself.
		// ie. MKeyboard_grave =  = /grave != ` = /quoteleft . (`)
		// -					
		MKeyboard_grave			=0xf040,
		MKeyboard_acute			=0xf080,
		MKeyboard_circumflex	=0xf0c0,
		MKeyboard_tilde			=0xf100,
		MKeyboard_macron		=0xf140,
		MKeyboard_breve			=0xf180,
		MKeyboard_dotaccent		=0xf1c0,
		MKeyboard_dieresis		=0xf200,
		// -
		MKeyboard_ring			=0xf280,
		MKeyboard_cedilla		=0xf2c0,
		// -
		MKeyboard_hungarumlaut	=0xf340,
		MKeyboard_ogonek		=0xf380,
		MKeyboard_caron			=0xf3c0,
		
		MKeyboard_sterling		=0xf400,
		MKeyboard_section,
		MKeyboard_arrowleft,
		MKeyboard_arrowup,
		MKeyboard_arrowright,
		MKeyboard_arrowdown,
		MKeyboard_degree,
		MKeyboard_plusminus,
		MKeyboard_divide,
		MKeyboard_onequarter,
		MKeyboard_onehalf,
		MKeyboard_threequarters,
		MKeyboard_OE,
		MKeyboard_oe,
		MKeyboard_germandbls,
	
	// following are control keys.
	// such control "character" is probably not in the UNICODE set, therefore
	// we'll code them with reserved codes.
	
		MKeyboard_F0		=0xfe00,	// waitting for actual UNICODE.
		MKeyboard_F1,
		MKeyboard_F2,
		MKeyboard_F3,
		MKeyboard_F4,
		MKeyboard_F5,
		MKeyboard_F6,
		MKeyboard_F7,
		MKeyboard_F8,
		MKeyboard_F9,
		// SEE: find the correct english translation of these keys:
		MKeyboard_Envoi,		// Send ?
		MKeyboard_Retour,		// Revert ?
		MKeyboard_Repetition,	// Repeat ?
		MKeyboard_Guide,		// Help ?
		MKeyboard_Annulation,	// Cancel ?
		MKeyboard_Sommaire,		// Menu ?
		MKeyboard_Correction,	// Correct ?
		MKeyboard_Suite,		// Next ?
		MKeyboard_ConnexionFin,	// Connection/End ?
		//
		MKeyboard_cursorLeft,
		MKeyboard_cursorUp,
		MKeyboard_cursorRight,
		MKeyboard_cursorDown,
		MKeyboard_deleteLine,
		MKeyboard_insertLine,
		MKeyboard_deleteChar,
		MKeyboard_insertMode,
		MKeyboard_overwriteMode,
		MKeyboard_home,
		MKeyboard_clearScreen,
		MKeyboard_LastCode		// excluded: this code is not valid.
	};

#ifdef MKeyboard_accents_macro
#define x_grave(l)			(MKeyboard_grave+(l)&0x3f)
#define x_acute(l)			(MKeyboard_acute+(l)&0x3f)
#define x_circumflex(l)		(MKeyboard_circumflex+(l)&0x3f)
#define x_tilde(l)			(MKeyboard_tilde+(l)&0x3f)
#define x_macron(l)			(MKeyboard_macron+(l)&0x3f)
#define x_breve(l)			(MKeyboard_breve+(l)&0x3f)
#define x_dotaccent(l)		(MKeyboard_dotaccent+(l)&0x3f)
#define x_dieresis(l)		(MKeyboard_dieresis+(l)&0x3f)
#define x_ring(l)			(MKeyboard_ring+(l)&0x3f)
#define x_cedilla(l)		(MKeyboard_cedilla+(l)&0x3f)
#define x_hungarumlaut(l)	(MKeyboard_hungarumlaut+(l)&0x3f)
#define x_ogonek(l)			(MKeyboard_ogonek+(l)&0x3f)
#define x_caron(l)			(MKeyboard_caron+(l)&0x3f)
#endif

#define MKeyboard_SUPER MModule
class MKeyboard:public MKeyboard_SUPER
{
public:

	CONSTRUCTOR(MKeyboard,(void))
	DESTRUCTOR(MKeyboard,(void))
	
// MObject methods:

	METHOD(MKeyboard,className,(void),const char*)

// MModule methods:

	// Normally, a keyboard has no need for a process method.

// MKeyboard methods:
	
	METHOD(MKeyboard,setExtended,(BOOLEAN extendedMode),void)
	METHOD(MKeyboard,extended,(void),BOOLEAN)
	
	METHOD(MKeyboard,setAsciiCursor,(BOOLEAN asciiCursor),void)
	METHOD(MKeyboard,asciiCursor,(void),BOOLEAN)
	
	METHOD(MKeyboard,setCapsLock,(BOOLEAN capsLock),void)
	METHOD(MKeyboard,capsLock,(void),BOOLEAN)
		/*
			Default setting is capsLock()=TRUE.
			capsLock() => [A-Za-z] codes are translated to [a-zA-Z].
			(Some Videotex servers don't understand low case characters).
		*/
		
		
	METHOD(MKeyboard,keyPressed,(UNICODE code),void)
		// Sends the sequence for the code to it's output.

	
	METHOD(MKeyboard,setProtocol,(MProtocol* nProtocol),void)
	
protected:
	CARD16			extendedFlag:1,
					asciiCursorFlag:1,
					capsLockFlag:1,
					lastKeyWasConnectionEnd:1,
					filler:12;
	MProtocol*		protocol;	// typically, it's the same as output.
};//MKeyboard.


#endif //MKeyboard.
