/******************************************************************************
FILE:				MMdHayes.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define a modem module via a Hayes modem.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1994/01/17 <PJB> Creation.
	1994/02/03 <PJB> Changed super class: now MMdViaSerial. Removed some 
					useless methods.
	1994/02/15 <PJB> Added protected fields.
	1994/02/22 <PJB> Added className method.
	1994/03/19 <PJB> Added some strings. Added MMdString class.
	1994/03/21 <PJB> Added AT string, and ATing state.
	1994/03/25 <PJB> Added parity inhibition option.
BUGS
	* The MMdString objects and the processInput scanner should be replaced
	  with a table driven scanner, with a scanner table builder embedded in 
	  to setString().
LEGAL
	Copyright Pascal J. Bourguignon 1994 - 1994
	All Rights Reserved.
******************************************************************************/
#ifndef __MMdHayes__
#define __MMdHayes__
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H
#include MMDVIASERIAL_HH
	

// public typedefs:

	typedef enum {						// default:
	// commands:
		MMdHayes_StrIndex_At,			// "AT\015
		MMdHayes_StrIndex_Reset,		// "ATZ0E0Q0V1\015"
		MMdHayes_StrIndex_PickUp,		// "ATH1O\015"
		MMdHayes_StrIndex_Dial,			// "ATD%s\015"
		MMdHayes_StrIndex_Escape,		// "+++"
	// responses from modem:
	// if you add strings, update the {First,Last}String macros in MMdHayes.cc.
		MMdHayes_StrIndex_Ok,			// "\015\012OK\015\012"
		MMdHayes_StrIndex_Connect,		// "\015\012CONNECT"
		MMdHayes_StrIndex_Ring,			// "\015\012RING\015\012"
		MMdHayes_StrIndex_NoCarrier,	// "\015\012NO CARRIER\015\012"
		MMdHayes_StrIndex_Error,		// "\015\012ERROR"
		MMdHayes_StrIndex_NoDialtone,	// "\015\012NO DIALTONE\015\012"
		MMdHayes_StrIndex_Busy,			// "\015\012BUSY\015\012"
		MMdHayes_StrIndex_Carrier,		// "\015\012CARRIER"
		MMdHayes_StrIndex_Count
	}					MMdHayes_StrIndexT;

#define MMdHayes_StrIndex_Min	MMdHayes_StrIndex_At
#define MMdHayes_StrIndex_Max	MMdHayes_StrIndex_Carrier


// protected typedefs (ie. clients must not use them):

	typedef enum {
		MMdHayes_State_Baby,
		MMdHayes_State_WaitBefore,
		MMdHayes_State_WaitAfter,
		MMdHayes_State_ATing,
		MMdHayes_State_Reseting,
		MMdHayes_State_Disconnected,
		MMdHayes_State_Connecting,
		MMdHayes_State_CarrierDetected,
		MMdHayes_State_ControlerSignaled
	}					MMdHayes_StateT;

	typedef enum {
		MMdString_Action_Recognize,	// => count=length
		MMdString_Action_Lock,		// => count=1
		MMdString_Action_Unlock		// => 1<=count<length
	}					MMdString_ActionT;
	
	
class MMdString
{
public:
	CONSTRUCTOR(MMdString,(void))
	DESTRUCTOR(MMdString,(void))
	
	METHOD(MMdString,setString,(const char* value),void)
		// NOTE: if analyze last returned MMdString_Action_Lock then
		//		you should send a reset after setString.
	METHOD(MMdString,getString,(void),const char*)
	METHOD(MMdString,length,(void),INT32)
	
	METHOD(MMdString,reset,(INT32 nBase),void)
	METHOD(MMdString,setBase,(INT32 nBase),void)
	METHOD(MMdString,analyze,(char token,INT16* count),MMdString_ActionT)
	METHOD(MMdString,lockedFrom,(void),INT32)
		// PRE: analyze last returned MMdString_Action_Lock.
	
protected:
	char*				string;
	INT16*				delta;
	INT16				sLength;
	INT16				index;	// [-1..length-1]; string[0..index] matched.
	INT32				lFrom;
	
	METHOD(MMdString,computeDelta,(void),void)
};//MMdString


#define MMdHayes_SUPER MMdViaSerial
class MMdHayes:public MMdHayes_SUPER
{
public:

	CONSTRUCTOR(MMdHayes,(void))
	DESTRUCTOR(MMdHayes,(void))

// MObject methods:

	METHOD(MMdHayes,className,(void),const char*)

// MModule methods:

// MModem methods:
	
	METHOD(MMdHayes,setMode,(MModem_ModeT newMode),void)
	METHOD(MMdHayes,dialAndConnect,(const char* number),void)
	METHOD(MMdHayes,connect,(void),void)
	METHOD(MMdHayes,disconnect,(void),void)
	METHOD(MMdHayes,startTurnAround,(void),void)
	
// MMdViaSerial methods:

	METHOD(MMdHayes,setSerial,(MSerial* nSerial),void)

// MHayes methods:

	METHOD(MMdHayes,setString,
						(MMdHayes_StrIndexT index,const char* value),void)
	METHOD(MMdHayes,getString,(MMdHayes_StrIndexT index),const char*)

// protected:
	
	METHOD(MMdHayes,hayesTimerElapsed,(INT32 reference),void)
	
protected:
	MMdHayes_StateT		hState;
	MMdString*			strings[MMdHayes_StrIndex_Count];
	MMdString*			locker;
	INT32				uFrom;
	MTimer*				hTimer;
	CARD32				carrierDelay;	//    80 ms (for Minitel).
	CARD32				okDelay;		//  6000 ms
	CARD32				connectDelay;	// 60000 ms
	CARD32				escapeDelay;	//  1500 ms
	
	
// MModem methods:
	METHOD(MMdHayes,processOutput,(const char* buffer,INT32 count),void)
	METHOD(MMdHayes,processInput,(const char* buffer,INT32 count),void)

// MHayes methods:

	METHOD(MMdHayes,enterState,(CARD8 nState),void)
		// enters the state nState.
	METHOD(MMdHayes,exitState,(void),void)
		// exits the current state (hState).
	METHOD(MMdHayes,receivedString,(MMdHayes_StrIndexT str),void)
		// process the event.
	METHOD(MMdHayes,repairState,(const char* methodName),void)
		// should never be needed, but errare humanum est.
};//MMdHayes.


#endif //MMdHayes.
