/******************************************************************************
FILE:				MModule.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define an abstract Minitel module.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/21 <PJB> Creation.
	1994/02/22 <PJB> Added the MObject super class. Added the className method.
	1994/03/23 <PJB> Separated setProtocol and setCharacterSet.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1993
	All Rights Reserved.
******************************************************************************/
#ifndef __MModule__
#define __MModule__
#include <mnames.h>
#include MOBJECT_HH

// enum = long !

	enum {
		MModule_Protocol_Videotex=			0x02,
		MModule_Protocol_Mixte=				0x03,
		MModule_Protocol_Telecomputing=		0x04
	};
	typedef CARD8				MModule_ProtocolT;
	/*
		Mixte is a special case of Videotex; we must have:
			MModule_Protocol_Videotex & MModule_Protocol_Mixte != 0.
	*/

	enum {
		MModule_Set_UsASCII,
		MModule_Set_FrenchASCII
	};
	typedef CARD8				MModule_CharacterSetT;


#define MModule_SUPER	MObject
class MModule:public MModule_SUPER
{
public:

	CONSTRUCTOR(MModule,(void))
	DESTRUCTOR(MModule,(void))
	
// MObject methods:

	METHOD(MModule,className,(void),const char*)

// MModule methods:

	METHOD(MModule,setOutput,(MModule* nOutput,INT32 nRef),void)
		/*
			nOuput is the output module for this module.
			nRef is the reference number of this module.
			the protocol module sets nRef=-2 for unattached modules.
		*/
	METHOD(MModule,process,(const char* buffer,INT32 count,INT32 ref),void)
		/*
			input routine for this module.
			ref is the reference number of the sending module.
			the protocol module sets ref to the reference of the receiver.
			some modules (modem, socket) accept ref=-1 from the serial line.
		*/
		
	METHOD(MModule,setProtocol,(MModule_ProtocolT nProto),void)
	METHOD(MModule,protocol,(void),MModule_ProtocolT)
	METHOD(MModule,setCharacterSet,(MModule_CharacterSetT nSet),void)
	METHOD(MModule,characterSet,(void),MModule_CharacterSetT)
		// set is valid only when protocol()=MModule_Protocol_Telecomputing

	METHOD(MModule,gotoPADX3Mode,(void),void)
	METHOD(MModule,padX3Mode,(void),BOOLEAN)
	
protected:
	MModule*				output;
	INT32					outputRef;
	MModule_ProtocolT		currentProtocol;
	MModule_CharacterSetT	currentCharacterSet;
	BOOLEAN					inPADX3Mode;	// can only go from FALSE to TRUE.
	
	METHOD(MModule,sendToOutput,(const char* buffer,INT32 count),void)
		// output routine.
		// = if(output!=NIL){output->process(buffer,count,outputRef);}
		
};//MModule.

#endif //MModule.
