/******************************************************************************
FILE:				MObject.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define a root class.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1994/02/22 <PJB> Creation.
	1994/02/24 <PJB> Added error reporting methods.
LEGAL
	Copyright Pascal J. Bourguignon 1994 - 1994
	All Rights Reserved.
******************************************************************************/
#ifndef __MObject__
#define __MObject__
extern "C"{
#include <stdarg.h>
}
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H

	typedef void (*MObject_ErrorPr)(void*,const char*,...);
	
	
	PROCEDURE(MObject_SetErrorHandler,
					(MObject_ErrorPr newHandler,void* newParameter),void)
	PROCEDURE(MObject_GetErrorHandler,
					(MObject_ErrorPr* curHandler,void** curParameter),void)
	PROCEDURE(MObject_DefaultErrorHandler,
					(void* parameter,const char* format,...),void)
		// fprintf(stderr,format,...);
		// NOTE: this may not be appropriate behaviour in your environment.


#ifdef MPW
#define MObject_SUPER SimpleObject
class MObject:public MObject_SUPER
#else
#define MObject_SUPER MObject
class MObject
#endif
{
public:
	CONSTRUCTOR(MObject,(void))
	DESTRUCTOR(MObject,(void))
// MObject methods:
	METHOD(MObject,className,(void),const char*)
	METHOD(MObject,error,(const char* format,...),void)
		// calls the current error handler adding the following prefix:
		// "error on %s[%p]:",className(),(void*)this
	METHOD(MObject,subclassResponsibility,(const char* methodName),void)
		// do error("method '%s' is subclass responsibility.",methodName);
	METHOD(MObject,notImplemented,(const char* methodName),void)
		// do error("method '%s' is not implemented.",methodName);

#undef ASSERT
#define ASSERT(expression) \
  ((expression)?(void)0:error("Assertion failed: (%s), file '%s', line %d.",\
  		 #expression,__FILE__,__LINE__))


};//MObject

#endif 
