/******************************************************************************
FILE:				MProtocol.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class defines the Minitel Protocole module.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/21 <PJB> Creation.
	1994/01/01 <PJB> Added the doThing methods.
	1994/02/22 <PJB> Added className method.
	1994/03/23 <PJB> Separated setProtocol and setCharacterSet.
	1994/03/25 <PJB> Added forceDisconnect, moved setModule to protected.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1993
	All Rights Reserved.
******************************************************************************/
#ifndef __MProtocol__
#define __MProtocol__
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H
#include MMODULE_HH
#include MSCREEN_HH
#include MKEYBOARD_HH
#include MMODEM_HH
#include MSOCKET_HH


#define MProtocol_MaxModules	(1<<3)	// =8. It must be a power of 2.
	
	typedef struct {
		MModule*				module;
		CARD8					receiverStatus;
		CARD8					diffusion:1,
								acknowledge:1,
								filler:6;
		CARD8					state;
		CARD8					count;
		char					buffer[6];
	}						MProtocol_ContextT;
	
#define MProtocol_SUPER MModule
class MProtocol:public MProtocol_SUPER,public MModemControler
{
public:

	CONSTRUCTOR(MProtocol,(void))
	DESTRUCTOR(MProtocol,(void))
	
// MModemControler methods (should not be called by other objects):
	METHOD(MProtocol,connectionFailed,(INT32 senderRef),void)
	METHOD(MProtocol,carrierDetected,(INT32 senderRef),void)
	METHOD(MProtocol,connected,(INT32 senderRef),void)
	METHOD(MProtocol,disconnected,(INT32 senderRef),void)
	METHOD(MProtocol,startTurnAround,(INT32 senderRef),void)
	METHOD(MProtocol,spurious,(INT32 senderRef),void)
	METHOD(MProtocol,turnedAround,(INT32 senderRef),void)

// MObject methods:

	METHOD(MProtocol,className,(void),const char*)

// MModule methods:

	METHOD(MProtocol,setProtocol,(MModule_ProtocolT nProto),void)
	METHOD(MProtocol,setCharacterSet,(MModule_CharacterSetT nSet),void)

	METHOD(MProtocol,setOutput,(MModule* nOutput,INT32 nRef),void)
		// NOP: the protocol objects haven't a single output module.
		
	METHOD(MProtocol,process,(const char* buffer,INT32 count,INT32 ref),void)
		// input routine.	

	METHOD(MProtocol,gotoPADX3Mode,(void),void)

// MProtocol methods:
	
	METHOD(MProtocol,setKeyboard,(MKeyboard* newKeyboard),void)
	METHOD(MProtocol,setScreen,(MScreen* newScreen),void)
	METHOD(MProtocol,setModem,(MModem* newModem),void)
	METHOD(MProtocol,setSocket,(MSocket* newSocket),void)
		// attach the specified module to this protocol.
		
	METHOD(MProtocol,keyboard,(void),MKeyboard*)
	METHOD(MProtocol,screen,(void),MScreen*)
	METHOD(MProtocol,modem,(void),MModem*)
	METHOD(MProtocol,socket,(void),MSocket*)
		// return the attached modules.

	METHOD(MProtocol,setRom,(const char* nRom),void)
		// PRE: strlen(nRom)=3
	
	METHOD(MProtocol,setLocalEcho,(BOOLEAN on),void)
	METHOD(MProtocol,localEcho,(void),BOOLEAN)
	
	METHOD(MProtocol,setInhibitSocket,(BOOLEAN on),void)
	METHOD(MProtocol,inhibitSocket,(void),BOOLEAN)
	
// SEE: check these are good primitives:
	METHOD(MProtocol,reset,(void),void)
	METHOD(MProtocol,setOpposed,(void),void)
	METHOD(MProtocol,setSlave,(void),void)
	METHOD(MProtocol,turnAround,(void),void)
	
	METHOD(MProtocol,forceDisconnect,(void),void)
		// double Connection/End key press.
		
protected:
	MProtocol_ContextT		context[MProtocol_MaxModules];
	BOOLEAN					modemIsOpposed;	// at connection.
	BOOLEAN					isConnected;
	BOOLEAN					alreadyTurnedAround;
	BOOLEAN					doingLocalEcho;
	BOOLEAN					isSocketInhibited;
	char					rom[5];
// MModule methods:

	METHOD(MProtocol,sendToOutput,(char* buffer,INT32 count),void)
		// NOP: the protocol objects haven't a single output module.

// MProtocol methods:
	METHOD(MProtocol,setModule,(MModule* newModule,INT16 mIndex),void)
		// BEWARE: setKeyboard does more things than simply setModule.
private:

// MProtocol methods:

	// SEE: could these methods be protected only ?
	METHOD(MProtocol,passAlong,
				(const char* buffer,INT32 count,INT32 sctx),void)
	METHOD(MProtocol,acknowledge,
				(const char* buffer,INT32 count,INT32 sctx),void)

	METHOD(MProtocol,doModemHangUp,(INT32 sctx),void)
	METHOD(MProtocol,doModemHangOn,(INT32 sctx),void)
	METHOD(MProtocol,doModemTurnAround1,(INT32 sctx),void)
	METHOD(MProtocol,doModemTurnAround2,(INT32 sctx),void)
	METHOD(MProtocol,doModemTurnAroundAck,(INT32 sctx),void)
	METHOD(MProtocol,doModemOpposition,(INT32 sctx),void)
	METHOD(MProtocol,doTerminalStatus,(INT32 sctx),void)
	METHOD(MProtocol,doFunctionningStatus,(INT32 sctx),void)
	METHOD(MProtocol,doSocketSpeedStatus,(INT32 sctx),void)
	METHOD(MProtocol,doProtocolStatus,(INT32 sctx),void)
	METHOD(MProtocol,doIdentification1,(INT32 sctx),void)
	METHOD(MProtocol,doIdentification2,(INT32 sctx),void)
	METHOD(MProtocol,doEnquireRam2,(INT32 sctx),void)
	METHOD(MProtocol,doEnquireRom,(INT32 sctx),void)
	METHOD(MProtocol,doReset,(INT32 sctx),void)
	METHOD(MProtocol,doTelecomputing,(INT32 sctx),void)
	METHOD(MProtocol,doMixte1,(INT32 sctx),void)
	METHOD(MProtocol,doMixte2,(INT32 sctx),void)
	METHOD(MProtocol,doShuntStatus,(INT32 sctx),void)
	METHOD(MProtocol,doDiffAck,(INT32 sctx),void)
	METHOD(MProtocol,doTransparence,(INT32 sctx),void)
	METHOD(MProtocol,doStartRollMode,(INT32 sctx),void)
	METHOD(MProtocol,doStartECP,(INT32 sctx),void)
	METHOD(MProtocol,doStartLowCaps,(INT32 sctx),void)
	METHOD(MProtocol,doStopRollMode,(INT32 sctx),void)
	METHOD(MProtocol,doStopECP,(INT32 sctx),void)
	METHOD(MProtocol,doStopLowCaps,(INT32 sctx),void)
	METHOD(MProtocol,doSocketSpeed,(INT32 sctx),void)
	METHOD(MProtocol,doModemSlave,(INT32 sctx),void)
	METHOD(MProtocol,doKeyboardStatus,(INT32 sctx),void)
	METHOD(MProtocol,doScreenHardCopy,(INT32 sctx),void)
	METHOD(MProtocol,doShunt,(INT32 sctx),void)
	METHOD(MProtocol,doKeyboardExtendedStart,(INT32 sctx),void)
	METHOD(MProtocol,doKeyboardCursorInC0Start,(INT32 sctx),void)
	METHOD(MProtocol,doKeyboardExtendedStop,(INT32 sctx),void)
	METHOD(MProtocol,doKeyboardCursorInC0Stop,(INT32 sctx),void)
	METHOD(MProtocol,doCursorPosition,(INT32 sctx),void)
	METHOD(MProtocol,pIdle,(INT32 sctx,char byte),INT32)
	METHOD(MProtocol,pEscaped,(INT32 sctx,char byte),INT32)
	METHOD(MProtocol,pPro1,(INT32 sctx,char byte),INT32)
	METHOD(MProtocol,pPro2,(INT32 sctx,char byte),INT32)
	METHOD(MProtocol,pPro3,(INT32 sctx,char byte),INT32)
	METHOD(MProtocol,pTransparent,(INT32 sctx,char byte),INT32)

};//MProtocol.

#endif //MProtocol.
