/******************************************************************************
FILE:				MScreen.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define an abstract Minitel modem module.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/21 <PJB> Creation.
	1994/02/22 <PJB> Added className method.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1993
	All Rights Reserved.
******************************************************************************/
#ifndef __MScreen__
#define __MScreen__
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H
extern "C"{
#include VTSCREEN_H
}
#include MMODULE_HH

class MView;	// forward


	typedef VTScreen_RectT			MScreen_RectT;
	typedef VTScreen_PixMapT		MScreen_PixMapT;
	typedef VTScreen_ResultT		MScreen_ResultT;
#define MScreen_Result_Null		VTScreen_Result_Null
#define MScreen_Result_Update	VTScreen_Result_Update
#define MScreen_Result_Beep		VTScreen_Result_Beep


	typedef enum {
		MScreen_Mode_Page,
		MScreen_Mode_Scroll
	}						MScreen_ModeT;
	

	typedef char			MScreen_HardCopyT[25][84];
			// each line is CR-LF-NUL-terminated.


	typedef enum {
		MScreen_ConnectedFlag_Local,
		MScreen_ConnectedFlag_LocalOpposed,
		MScreen_ConnectedFlag_Connecting,
		MScreen_ConnectedFlag_Connected
	}						MScreen_ConnectedFlagT;

	

#define MScreen_SUPER MModule
class MScreen:public MScreen_SUPER
{
public:

	CONSTRUCTOR(MScreen,(void))
	DESTRUCTOR(MScreen,(void))
	
// MObject methods:

	METHOD(MScreen,className,(void),const char*)
	
// MModule methods:
	
// MScreen methods:
	METHOD(MScreen,setClut,(VTScreen_ClutT* newClut),void)
	METHOD(MScreen,getClut,(VTScreen_ClutT* oldClut),void)
	
	METHOD(MScreen,setView,(MView* newView),void)
	METHOD(MScreen,view,(void),MView*)
	
	METHOD(MScreen,setPageMode,(BOOLEAN on),void)
	METHOD(MScreen,pageMode,(void),BOOLEAN)
		/*
			pageMode()=TRUE  <=> page   mode.
			pageMode()=FALSE <=> scroll mode.
		*/
	

	METHOD(MScreen,setEightyColumns,(BOOLEAN on),void)
	METHOD(MScreen,eightyColumns,(void),BOOLEAN)
		/*
			When protocol()=MModule_Protocol_Telecomputing, the screen
			can have 40 or 80 columns (default is 80). 
		*/

	METHOD(MScreen,getHardCopy,(MModule_CharacterSetT set,MScreen_HardCopyT* hc),void)
	
	METHOD(MScreen,getCursor,(INT16* line,INT16* column),void)
		// line IN [0..24], column IN [1..40] or [1..80]
		
		
	METHOD(MScreen,setFlagInhibitSocket,(BOOLEAN on),void)
	METHOD(MScreen,flagInhibitSocket,(void),BOOLEAN)
	
	METHOD(MScreen,setFlagHardCopy,(BOOLEAN on),void)
	METHOD(MScreen,flagHardCopy,(void),BOOLEAN)

	METHOD(MScreen,setFlagConnected,(MScreen_ConnectedFlagT flag),void)
	METHOD(MScreen,flagConnected,(void),MScreen_ConnectedFlagT)
	
	METHOD(MScreen,process,(const char* buffer,INT32 count,INT32 ref),void)
	
	METHOD(MScreen,pixMap,(void),MScreen_PixMapT*)
	METHOD(MScreen,getUpdate,(INT32 time,
				MScreen_RectT* rect1,MScreen_RectT* rect2),MScreen_ResultT)
	METHOD(MScreen,setUpdated,(void),void)

protected:
	INT16					flags[4];		// in columns 36 .. 39
	VTScreen_T				screen;
	MView*					sView;
	METHOD(MScreen,plotFlags,(void),void)
};//MScreen.


#endif //MScreen.
