/******************************************************************************
FILE:				MSocket.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define an abstract Minitel socket module.
	Generaly, there is no socket in emulators, but this emulator library
	allows for a socket; this can be implemented at will, as a virtual
	socket (file), or a real serial line port, or anything.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/22 <PJB> Creation.
	1994/01/17 <PJB> Promoted speed to INT32 from INT16.
	1994/02/22 <PJB> Added className method.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1993
	All Rights Reserved.
******************************************************************************/
#ifndef __MSocket__
#define __MSocket__
#include <mnames.h>
#include MMODULE_HH
#include MSERIAL_HH
#include BCTYPES_H
#include BCINTERFACE_H

#define MSocket_SUPER MModule
class MSocket:public MSocket_SUPER
{
public:

	CONSTRUCTOR(MSocket,(void))
	DESTRUCTOR(MSocket,(void))
	
// MObject methods:

	METHOD(MSocket,className,(void),const char*)

// MSocket methods:
	
	METHOD(MSocket,setSerial,(MSerial* nSerial),void)
	METHOD(MSocket,serial,(void),MSerial*)

	METHOD(MSocket,setSpeed,(INT32 nSendSpeed,INT32 nReceiveSpeed),void)
	METHOD(MSocket,getSpeed,(INT32* cSendSpeed,INT32* cReceiveSpeed),void)
		/*
			For Minitel1B, only 300-300, 1200-1200 and 4800-4800 b/s speeds
			are valid.
			Changing the speed from 1200-1200, also changes some 
			STURM-related functionnalities. Cf STUM-1B 2.5.3.2.1 p.131
		*/
	
	
	METHOD(MSocket,process,(const char* buffer,INT32 count,INT32 ref),void)
		/*
			if ref = serial line ref then
				check and filer parity bits and sentToOutput
			 else add parity bits and send to serial line.
		*/
		
protected:
	MSerial*			serialLine;
	INT32				sendSpeed;
	INT32				receiveSpeed;
};//MSocket.


#endif //MSocket.
