/******************************************************************************
FILE:				MTimer.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class defines objects that call synchronously their handler 
	after a delay has elapsed.
	Calling synchronously means the handler is called at a time 
	where any message to a GUI object can be sent. With most GUIs, this means 
	not at interrupt time, but during the GUI event loop.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1994/02/10 <PJB> Creation.
	1994/02/21 <PJB> Removed 'class TimerDelegate': it does not work when
					a subclass wants to multiply inherit a same class than
					one of its superclass. Replaced it with the handler 
					routine.
	1994/02/22 <PJB> Added className method.
	1994/03/16 <PJB> Added MObject super class (missed it last time).
LEGAL
	Copyright Pascal J. Bourguignon 1994 - 1994
	All rights reserved.
******************************************************************************/
#ifndef __MTimer__
#define __MTimer__
#include <mnames.h>
#include MOBJECT_HH
#include BCINTERFACE_H
#include BCTYPES_H



typedef void (*MTimerPr)(void*,INT32);

#define MTimerHandler(procname,class,method)	\
	void procname(void* obj,INT32 ref) { ((class*)obj)->method(ref); }

#define MTimer_SUPER MObject
class MTimer:public MTimer_SUPER
{
public:

	CONSTRUCTOR(MTimer,(void))
	DESTRUCTOR(MTimer,(void))

// MObject methods:

	METHOD(MTimer,className,(void),const char*)

// MTimer methods:

	METHOD(MTimer,setHandler,(MTimerPr nHandler,void* nObj,INT32 nRef),void)
	METHOD(MTimer,getHandler,(MTimerPr* handlr,void** obj,INT32* ref),void)
	
	METHOD(MTimer,start,(CARD32 milliseconds),void)
		/*
			DO:			when at least milliseconds have elapsed, 
						call the handler.
			NOTE:		start(0) will call the handler as soon as possible.
		*/

	METHOD(MTimer,cancel,(void),void)
		/*
			DO:			cancel calling the handler.
		*/

//protected:

	METHOD(MTimer,elapsed,(void),void)
		/*
			DO:			if(handler!=NIL){handler(object,reference);}
			NOTE:		this method must be called at event loop time 
						when the implementation-specific (GUI-) event
						occurs.
		*/
		
protected:
	MTimerPr				handler;
	void*					object;
	INT32					reference;
private:
	void*					opaque;
};//MTimer

#endif

